use serde::{Deserialize, Serialize};
use serde_json::json;
use sqlx::FromRow;

use crate::get_pg;

use super::track::Track;

#[derive(Debug, Clone, Serialize, Deserialize, FromRow)]
pub struct Artist {
    pub id: uuid::Uuid,
    pub name: String,
}

impl Artist {
    pub async fn create(name: &str) -> Self {
        sqlx::query_as("INSERT INTO artist (name) VALUES ($1) RETURNING id, name")
            .bind(name)
            .fetch_one(get_pg!())
            .await
            .unwrap()
    }

    /// Gets the image of an artist or `None` if it can't be found.
    ///
    /// This function gets a track from the artist. It then expects the folder structure to be `Artist/Album/Track.ext` and searches for an image file named `artist` in the artist folder.
    pub async fn get_image_of(id: &uuid::Uuid) -> Option<String> {
        // todo : fix
        let track_path = Track::find_one(doc! { "artist_id": reference_of!(Artist, id)})
            .await?
            .path;
        let track_path = std::path::Path::new(&track_path);

        let artist_path = track_path.parent()?.parent()?;

        for ext in ["png", "jpg", "jpeg", "avif", "webp"] {
            let cover_file = artist_path.join(format!("artist.{ext}"));

            if cover_file.exists() {
                return Some(cover_file.to_str().unwrap().to_string());
            }
        }

        None
    }
}

impl Artist {
    async fn api(&self) -> serde_json::Value {
        json!({
            "id": &self.id,
            "name": &self.name,
            "image": if Artist::get_image_of(&self.id).await.is_some() {
                Some(format!("/artist/{}/image", self.id))
            } else {
                None
            }
        })
    }
}
